@@ -1,7 +1,7 @@
 /* gcal-window.c
  *
  * Copyright (C) 2015 Erick Pérez Castellanos <erickpc@gnome.org>
- * Copyright (C) 2014 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
+ * Copyright (C) 2014-2020 Georges Basile Stavracas Neto <georges.stavracas@gmail.com>
  *
  * gnome-calendar is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -30,6 +30,8 @@
 #include "gcal-month-view.h"
 #include "gcal-quick-add-popover.h"
 #include "gcal-search-button.h"
+#include "gcal-timeline.h"
+#include "gcal-timeline-subscriber.h"
 #include "gcal-view.h"
 #include "gcal-weather-settings.h"
 #include "gcal-week-view.h"
@@ -121,6 +123,7 @@ struct _GcalWindow
   /* day, week, month, year, list */
   GtkWidget          *views [6];
   GtkWidget          *edit_dialog;
+  gboolean            subscribed;
 
   GcalContext        *context;
   GcalWindowView      active_view;
@@ -233,6 +236,23 @@ update_today_button_sensitive (GcalWindow *window)
   GCAL_EXIT;
 }
 
+static void
+maybe_add_subscribers_to_timeline (GcalWindow *self)
+{
+  GcalTimeline *timeline;
+
+  if (self->subscribed)
+    return;
+
+
+  timeline = gcal_manager_get_timeline (gcal_context_get_manager (self->context));
+  gcal_timeline_add_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->week_view));
+  gcal_timeline_add_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->month_view));
+  gcal_timeline_add_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->year_view));
+
+  self->subscribed = TRUE;
+}
+
 static void
 update_active_date (GcalWindow *window,
                     GDateTime  *new_date)
@@ -248,15 +268,12 @@ update_active_date (GcalWindow *window,
   gcal_set_date_time (&window->active_date, new_date);
 
   for (i = GCAL_WINDOW_VIEW_WEEK; i <= GCAL_WINDOW_VIEW_YEAR; i++)
-    {
-      GcalView *view = GCAL_VIEW (window->views[i]);
-
-      gcal_view_set_date (view, new_date);
-      gcal_view_update_subscription (view);
-    }
+    gcal_view_set_date (GCAL_VIEW (window->views[i]), new_date);
 
   update_today_button_sensitive (window);
 
+  maybe_add_subscribers_to_timeline (window);
+
   GCAL_EXIT;
 }
 
@@ -962,6 +979,22 @@ gcal_window_get_property (GObject    *object,
  * GtkWidget overrides
  */
 
+static void
+gcal_window_destroy (GtkWidget *widget)
+{
+  GcalTimeline *timeline;
+  GcalWindow *self;
+
+  self = GCAL_WINDOW (widget);
+
+  timeline = gcal_manager_get_timeline (gcal_context_get_manager (self->context));
+  gcal_timeline_remove_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->week_view));
+  gcal_timeline_remove_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->month_view));
+  gcal_timeline_remove_subscriber (timeline, GCAL_TIMELINE_SUBSCRIBER (self->year_view));
+
+  GTK_WIDGET_CLASS (gcal_window_parent_class)->destroy (widget);
+}
+
 static gboolean
 gcal_window_configure_event (GtkWidget         *widget,
                              GdkEventConfigure *event)
@@ -1004,6 +1037,7 @@ gcal_window_class_init (GcalWindowClass *klass)
 
   widget_class = GTK_WIDGET_CLASS (klass);
   widget_class->configure_event = gcal_window_configure_event;
+  widget_class->destroy = gcal_window_destroy;
 
 
   properties[PROP_ACTIVE_DATE] = g_param_spec_boxed ("active-date",
