@@ -575,12 +575,12 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
 
       for (j = 0; widgets_data && j < widgets_data->len; j++)
         {
+          g_autoptr (GDateTime) event_start = NULL;
+          g_autoptr (GDateTime) event_end = NULL;
           GtkStyleContext *context;
           GtkAllocation child_allocation;
           GtkWidget *event_widget;
           GcalRange *event_range;
-          GDateTime *event_start;
-          GDateTime *event_end;
           ChildData *data;
           GtkBorder margin;
           guint64 events_at_range;
@@ -594,8 +594,8 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
           data =  g_ptr_array_index (widgets_data, j);
           event_widget = data->widget;
           event_range = gcal_event_get_range (data->event);
-          event_start = gcal_event_get_date_start (data->event);
-          event_end = gcal_event_get_date_end (data->event);
+          event_start = g_date_time_to_local (gcal_event_get_date_start (data->event));
+          event_end = g_date_time_to_local (gcal_event_get_date_end (data->event));
           context = gtk_widget_get_style_context (event_widget);
 
           /* The total number of events available in this range */
@@ -900,11 +900,12 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
                           gint            y,
                           guint           time)
 {
+  g_autoptr (GDateTime) week_start = NULL;
+  g_autoptr (GDateTime) dnd_date = NULL;
+  g_autoptr (GDateTime) new_end = NULL;
+  g_autoptr (GcalEvent) changed_event = NULL;
   GcalRecurrenceModType mod;
   GcalWeekGrid *self;
-  g_autoptr (GDateTime) week_start;
-  g_autoptr (GDateTime) dnd_date;
-  g_autoptr (GDateTime) new_end;
   GcalCalendar *calendar;
   GtkWidget *event_widget;
   GcalEvent *event;
@@ -918,9 +919,6 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
   event_widget = gtk_drag_get_source_widget (context);
 
   mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
-  week_start = NULL;
-  dnd_date = NULL;
-  new_end = NULL;
 
   if (!GCAL_IS_EVENT_WIDGET (event_widget))
     return FALSE;
@@ -937,9 +935,10 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
     }
 
   event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (event_widget));
-  calendar = gcal_event_get_calendar (event);
+  changed_event = gcal_event_new_from_event (event);
+  calendar = gcal_event_get_calendar (changed_event);
 
-  if (gcal_event_has_recurrence (event) &&
+  if (gcal_event_has_recurrence (changed_event) &&
       !ask_recurrence_modification_type (widget, &mod, calendar))
     {
       goto out;
@@ -952,23 +951,23 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
    * Calculate the diff between the dropped cell and the event's start date,
    * so we can update the end date accordingly.
    */
-  timespan = g_date_time_difference (gcal_event_get_date_end (event), gcal_event_get_date_start (event));
+  timespan = g_date_time_difference (gcal_event_get_date_end (changed_event), gcal_event_get_date_start (changed_event));
 
   /*
    * Set the event's start and end dates. Since the event may have a
    * NULL end date, so we have to check it here
    */
-  gcal_event_set_all_day (event, FALSE);
-  gcal_event_set_date_start (event, dnd_date);
+  gcal_event_set_all_day (changed_event, FALSE);
+  gcal_event_set_date_start (changed_event, dnd_date);
 
 
   /* Setup the new end date */
   new_end = g_date_time_add (dnd_date, timespan);
-  gcal_event_set_date_end (event, new_end);
+  gcal_event_set_date_end (changed_event, new_end);
 
   /* Commit the changes */
 
-  gcal_manager_update_event (gcal_context_get_manager (self->context), event, mod);
+  gcal_manager_update_event (gcal_context_get_manager (self->context), changed_event, mod);
 
 out:
   /* Cancel the DnD */
