@@ -21,6 +21,7 @@
 #include "gcal-debug.h"
 #include "gcal-enums.h"
 #include "gcal-event-widget.h"
+#include "gcal-timeline-subscriber.h"
 #include "gcal-utils.h"
 #include "gcal-view.h"
 #include "gcal-week-header.h"
@@ -61,7 +62,7 @@ static void          schedule_position_scroll                    (GcalWeekView
 
 static void          gcal_view_interface_init                    (GcalViewInterface  *iface);
 
-static void          gcal_data_model_subscriber_interface_init   (ECalDataModelSubscriberInterface *iface);
+static void          gcal_timeline_subscriber_interface_init     (GcalTimelineSubscriberInterface *iface);
 
 enum
 {
@@ -74,8 +75,8 @@ enum
 
 G_DEFINE_TYPE_WITH_CODE (GcalWeekView, gcal_week_view, GTK_TYPE_BOX,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
-                         G_IMPLEMENT_INTERFACE (E_TYPE_CAL_DATA_MODEL_SUBSCRIBER,
-                                                gcal_data_model_subscriber_interface_init));
+                         G_IMPLEMENT_INTERFACE (GCAL_TYPE_TIMELINE_SUBSCRIBER,
+                                                gcal_timeline_subscriber_interface_init));
 
 /* Callbacks */
 static void
@@ -198,6 +199,8 @@ gcal_week_view_set_date (GcalView  *view,
 
   schedule_position_scroll (self);
 
+  gcal_timeline_subscriber_range_changed (GCAL_TIMELINE_SUBSCRIBER (view));
+
   GCAL_EXIT;
 }
 
@@ -227,28 +230,6 @@ gcal_week_view_clear_marks (GcalView *view)
   gcal_week_grid_clear_marks (GCAL_WEEK_GRID (self->week_grid));
 }
 
-static void
-gcal_week_view_update_subscription (GcalView *view)
-{
-  g_autoptr (GDateTime) date_start = NULL;
-  g_autoptr (GDateTime) date_end = NULL;
-  GcalWeekView *self;
-  time_t range_start;
-  time_t range_end;
-
-  self = GCAL_WEEK_VIEW (view);
-  date_start = gcal_date_time_get_start_of_week (self->date);
-  range_start = g_date_time_to_unix (date_start);
-
-  date_end = gcal_date_time_get_end_of_week (self->date);
-  range_end = g_date_time_to_unix (date_end);
-
-  gcal_manager_set_subscriber (gcal_context_get_manager (self->context),
-                               E_CAL_DATA_MODEL_SUBSCRIBER (self),
-                               range_start,
-                               range_end);
-}
-
 static GDateTime*
 gcal_week_view_get_next_date (GcalView *view)
 {
@@ -275,7 +256,6 @@ gcal_view_interface_init (GcalViewInterface *iface)
   iface->set_date = gcal_week_view_set_date;
   iface->get_children_by_uuid = gcal_week_view_get_children_by_uuid;
   iface->clear_marks = gcal_week_view_clear_marks;
-  iface->update_subscription = gcal_week_view_update_subscription;
   iface->get_next_date = gcal_week_view_get_next_date;
   iface->get_previous_date = gcal_week_view_get_previous_date;
 }
@@ -334,23 +314,28 @@ update_hours_sidebar_size (GcalWeekView *self)
   g_object_unref (layout);
 }
 
+/*
+ * GcalTimelineSubscriber iface
+ */
 
-/* ECalDataModelSubscriber implementation */
+static GcalRange*
+gcal_week_view_get_range (GcalTimelineSubscriber *subscriber)
+{
+  GcalWeekView *self = GCAL_WEEK_VIEW (subscriber);
+
+  return gcal_range_new_take (gcal_date_time_get_start_of_week (self->date),
+                              gcal_date_time_get_end_of_week (self->date),
+                              GCAL_RANGE_DEFAULT);
+}
+
 static void
-gcal_week_view_component_added (ECalDataModelSubscriber *subscriber,
-                                ECalClient              *client,
-                                ECalComponent           *comp)
+gcal_week_view_add_event (GcalTimelineSubscriber *subscriber,
+                          GcalEvent              *event)
 {
   GcalWeekView *self = GCAL_WEEK_VIEW (subscriber);
-  g_autoptr (GcalEvent) event = NULL;
-  GcalCalendar *calendar;
 
   GCAL_ENTRY;
 
-  calendar = gcal_manager_get_calendar_from_source (gcal_context_get_manager (self->context),
-                                                    e_client_get_source (E_CLIENT (client)));
-  event = gcal_event_new (calendar, comp, NULL);
-
   if (gcal_event_is_multiday (event) || gcal_event_get_all_day (event))
     gcal_week_header_add_event (GCAL_WEEK_HEADER (self->header), event);
   else
@@ -360,65 +345,45 @@ gcal_week_view_component_added (ECalDataModelSubscriber *subscriber,
 }
 
 static void
-gcal_week_view_component_modified (ECalDataModelSubscriber *subscriber,
-                                   ECalClient              *client,
-                                   ECalComponent           *comp)
+gcal_week_view_update_event (GcalTimelineSubscriber *subscriber,
+                             GcalEvent              *event)
 {
   GcalWeekView *self = GCAL_WEEK_VIEW (subscriber);
-  GcalWeekHeader *header;
-  gchar *uuid;
+  const gchar *event_id;
 
   GCAL_ENTRY;
 
-  header = GCAL_WEEK_HEADER (self->header);
+  event_id = gcal_event_get_uid (event);
+  gcal_week_header_remove_event (GCAL_WEEK_HEADER (self->header), event_id);
+  gcal_week_grid_remove_event (GCAL_WEEK_GRID (self->week_grid), event_id);
 
-  uuid = get_uuid_from_component (e_client_get_source (E_CLIENT (client)), comp);
-
-  gcal_week_header_remove_event (header, uuid);
-  gcal_week_grid_remove_event (GCAL_WEEK_GRID (self->week_grid), uuid);
-
-  gcal_week_view_component_added (subscriber, client, comp);
-
-  g_free (uuid);
+  gcal_week_view_add_event (subscriber, event);
 
   GCAL_EXIT;
 }
 
 static void
-gcal_week_view_component_removed (ECalDataModelSubscriber *subscriber,
-                                  ECalClient              *client,
-                                  const gchar             *uid,
-                                  const gchar             *rid)
+gcal_week_view_remove_event (GcalTimelineSubscriber *subscriber,
+                             GcalEvent              *event)
 {
   GcalWeekView *self = GCAL_WEEK_VIEW (subscriber);
-  ESource *source;
-  gchar *uuid;
+  const gchar *uuid = gcal_event_get_uid (event);
 
   GCAL_ENTRY;
 
-  source = e_client_get_source (E_CLIENT (client));
-
-  if (rid != NULL)
-    uuid = g_strdup_printf ("%s:%s:%s", e_source_get_uid (source), uid, rid);
-  else
-    uuid = g_strdup_printf ("%s:%s", e_source_get_uid (source), uid);
-
   gcal_week_header_remove_event (GCAL_WEEK_HEADER (self->header), uuid);
   gcal_week_grid_remove_event (GCAL_WEEK_GRID (self->week_grid), uuid);
 
-  g_free (uuid);
-
   GCAL_EXIT;
 }
 
 static void
-gcal_week_view_freeze (ECalDataModelSubscriber *subscriber)
-{
-}
-
-static void
-gcal_week_view_thaw (ECalDataModelSubscriber *subscriber)
+gcal_timeline_subscriber_interface_init (GcalTimelineSubscriberInterface *iface)
 {
+  iface->get_range = gcal_week_view_get_range;
+  iface->add_event = gcal_week_view_add_event;
+  iface->update_event = gcal_week_view_update_event;
+  iface->remove_event = gcal_week_view_remove_event;
 }
 
 static gboolean
@@ -530,16 +495,6 @@ gcal_week_view_draw_hours (GcalWeekView *self,
   return FALSE;
 }
 
-static void
-gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
-{
-  iface->component_added = gcal_week_view_component_added;
-  iface->component_modified = gcal_week_view_component_modified;
-  iface->component_removed = gcal_week_view_component_removed;
-  iface->freeze = gcal_week_view_freeze;
-  iface->thaw = gcal_week_view_thaw;
-}
-
 static void
 gcal_week_view_finalize (GObject       *object)
 {
@@ -547,7 +502,7 @@ gcal_week_view_finalize (GObject       *object)
 
   self = GCAL_WEEK_VIEW (object);
 
-  g_clear_pointer (&self->date, g_free);
+  g_clear_pointer (&self->date, g_date_time_unref);
 
   g_clear_object (&self->context);
 
